@@ -9,18 +9,21 @@ from changeo import __version__, __date__
 
 # Imports
 import os
-import sys
+import random
+import subprocess
+import multiprocessing as mp
 from argparse import ArgumentParser
 from collections import OrderedDict
 from textwrap import dedent
 from time import time
 from Bio.Seq import Seq
+from functools import partial
 
 # Presto and changeo imports
 from presto.Defaults import default_out_args
 from presto.IO import  printLog, printMessage, printWarning, printError, printDebug
 from changeo.Defaults import default_format
-from changeo.IO import splitName, getDbFields, getFormatOperators, getOutputHandle
+from changeo.IO import splitName, getDbFields, getFormatOperators, getOutputHandle, getOutputName
 from changeo.Alignment import getRegions
 from changeo.Commandline import CommonHelpFormatter, checkArgs, getCommonArgParser, parseCommonArgs
 
@@ -110,7 +113,7 @@ def checkFrameShifts(receptor, oqpos, ospos, log, debug):
     return frameshifts
 
 
-def findAndMask(receptor, scodons, qcodons, spos, s_end, qpos, log, debug, recursive):
+def findAndMask(receptor, scodons, qcodons, spos, s_end, qpos, log, debug, recursive=False):
     """
     Find and mask split codons
 
@@ -123,7 +126,7 @@ def findAndMask(receptor, scodons, qcodons, spos, s_end, qpos, log, debug, recur
       qpos (int): starting position of input sequence in IMGT sequence
       log (dict): log of information for each sequence
       debug (bool): print debugging statements?
-
+      recursive (bool): was this function called recursively?
     """
     frameshifts = 0
     while spos < s_end and qpos < len(qcodons):
@@ -333,6 +336,7 @@ def deduplicate(useqs, receptors, log=None, meta_data=None, delim=":"):
       receptors (dict): receptors within a clone (index is value in useqs dict).
       log (collections.OrderedDict): log of sequence errors.
       meta_data (str): Field to append to sequence IDs. Splits identical sequences with different meta_data.
+      meta_data (str): Field to append to sequence IDs. Splits identical sequences with different meta_data.
       delim (str): delimited to use when appending meta_data.
 
     Returns:
@@ -357,8 +361,12 @@ def deduplicate(useqs, receptors, log=None, meta_data=None, delim=":"):
             rj = receptors[useqs[kj]]
             dist = unAmbigDist(ski, skj, True)
             m_match = True
-            if meta_data is not None and meta_data[0] != "DUPCOUNT":
-                m_match = ri.getField(meta_data[0]) == rj.getField(meta_data[0])
+            if meta_data is not None:
+                matches = 0
+                for m in meta_data:
+                    if ri.getField(m) == rj.getField(m) and m != "DUPCOUNT":
+                        matches += 1
+                m_match = (matches == len(meta_data))
             if dist == 0 and m_match:
                 ncounti = ki.count("A") + ki.count("T") + ki.count("G") + ki.count("C")
                 ncountj = kj.count("A") + kj.count("T") + kj.count("G") + kj.count("C")
@@ -417,6 +425,44 @@ def hasPTC(sequence):
     return -1
 
 
+
+def rmCDR3(sequences, clones):
+    """
+    Remove CDR3 from all sequences and germline of a clone
+
+     Arguments:
+       sequences (list): list of sequences in clones.
+       clones (list): list of Receptor objects.
+    """
+
+    for i in range(0,len(sequences)):
+        imgtar = clones[i].getField("imgtpartlabels")
+        germline = clones[i].getField("germline_imgt_d_mask")
+        nseq = []
+        nimgtar = []
+        ngermline = []
+        ncdr3 = 0
+        #print("imgtarlen: " + str(len(imgtar)))
+        #print("seqlen: " + str(len(sequences[i])))
+        #print("germline: " + str(len(germline)))
+        #if len(germline) < len(sequences[i]):
+        #    print("\n" + str(clones[i].sequence_id))
+        #    print("\n " + str((sequences[i])) )
+        #    print("\n" + str((germline)))
+        for j in range(0,len(imgtar)):
+            if imgtar[j] != 108:
+                nseq.append(sequences[i][j])
+                if j < len(germline):
+                    ngermline.append(germline[j])
+                nimgtar.append(imgtar[j])
+            else:
+                ncdr3 += 1
+        clones[i].setField("imgtpartlabels",nimgtar)
+        clones[i].setField("germline_imgt_d_mask", "".join(ngermline))
+        sequences[i] = "".join(nseq)
+        #print("Length: " + str(ncdr3))
+
+
 def characterizePartitionErrors(sequences, clones, meta_data):
     """
     Characterize potential mismatches between IMGT labels within a clone
@@ -435,8 +481,8 @@ def characterizePartitionErrors(sequences, clones, meta_data):
     nseqs = len(sequences)
     imgtar = clones[0].getField("imgtpartlabels")
     germline = clones[0].getField("germline_imgt_d_mask")
-
     correctseqs = False
+
     for seqi in range(0, len(sequences)):
         i = sequences[seqi]
         if len(i) != sites or len(clones[seqi].getField("imgtpartlabels")) != len(imgtar):
@@ -498,12 +544,10 @@ def characterizePartitionErrors(sequences, clones, meta_data):
         germline = germline + "N" * (seqdiff)
 
     if sites % 3 != 0:
-        printError("number of sites must be divisible by 3! len: %d, clone: %s , seq: %s" %(len(sequences[0]),\
-        clones[0].clone,sequences[0]))
-
+        printError("number of sites must be divisible by 3! len: %d, clone: %s , id: %s, seq: %s" %(len(sequences[0]),\
+        clones[0].clone,clones[0].sequence_id,sequences[0]))
     return imgtar, germline, sites, nseqs
 
-
 def outputSeqPartFiles(out_dir, useqs_f, meta_data, clones, collapse, nseqs, delim, newgerm, conseqs, duplicate, imgt):
     """
     Create intermediate sequence alignment and partition files for IgPhyML output
@@ -513,7 +557,7 @@ def outputSeqPartFiles(out_dir, useqs_f, meta_data, clones, collapse, nseqs, del
         useqs_f (dict): unique sequences mapped to ids.
         meta_data (str): Field to append to sequence IDs. Splits identical sequences with different meta_data.
         clones (list) : list of receptor objects.
-        collpase (bool) : deduplicate sequences.
+        collapse (bool) : deduplicate sequences.
         nseqs (int): number of sequences.
         delim (str) : delimiter for extracting metadata from ID.
         newgerm (str) : modified germline of clonal lineage.
@@ -521,6 +565,11 @@ def outputSeqPartFiles(out_dir, useqs_f, meta_data, clones, collapse, nseqs, del
         duplicate (bool) : duplicate sequence if only one in a clone.
         imgt (list) : IMGT numbering of clonal positions .
     """
+
+    # bootstrap these data if desired
+    lg = len(newgerm)
+    sites = range(0, lg)
+
     transtable = clones[0].sequence_id.maketrans(" ", "_")
     outfile = os.path.join(out_dir, "%s.fasta" % clones[0].clone)
     with open(outfile, "w") as clonef:
@@ -530,32 +579,37 @@ def outputSeqPartFiles(out_dir, useqs_f, meta_data, clones, collapse, nseqs, del
                 cid = ""
                 if meta_data is not None:
                     seq, cid = seq_f.split(delim)
-                    clones[num].setField(meta_data[0], clones[num].getField(meta_data[0]).replace(":", "_"))
-                    cid = delim + str(clones[num].getField(meta_data[0]))
+                    cid = delim + cid.replace(":", "_")
                 sid = clones[num].sequence_id.translate(transtable) + cid
-                clonef.write(">%s\n%s\n" % (sid, seq.replace(".", "-")))
+                clonef.write(">%s\n%s\n" % (sid.replace(":","-"), seq.replace(".", "-")))
                 if len(useqs_f) == 1 and duplicate:
                     if meta_data is not None:
                         if meta_data[0] == "DUPCOUNT":
                             cid = delim + "0"
                     sid = clones[num].sequence_id.translate(transtable) + "_1" + cid
-                    clonef.write(">%s\n%s\n" % (sid, seq.replace(".", "-")))
+                    clonef.write(">%s\n%s\n" % (sid.replace(":","-"), seq.replace(".", "-")))
         else:
             for j in range(0, nseqs):
                 cid = ""
                 if meta_data is not None:
-                    clones[j].setField(meta_data[0], clones[j].getField(meta_data[0]).replace(":", "_"))
-                    cid = delim+str(clones[j].getField(meta_data[0]))
+                    meta_data_list = []
+                    for m in meta_data:
+                        meta_data_list.append(clones[j].getField(m).replace(":", "_"))
+                    cid = delim + str(delim.join(meta_data_list))
                 sid = clones[j].sequence_id.translate(transtable) + cid
-                clonef.write(">%s\n%s\n" % (sid, conseqs[j].replace(".", "-")))
+                clonef.write(">%s\n%s\n" % (sid.replace(":","-"), conseqs[j].replace(".", "-")))
                 if nseqs == 1 and duplicate:
                     if meta_data is not None:
                         if meta_data[0] == "DUPCOUNT":
                             cid = delim + "0"
                     sid = clones[j].sequence_id.translate(transtable)+"_1" + cid
-                    clonef.write(">%s\n%s\n" % (sid, conseqs[j].replace(".", "-")))
+                    clonef.write(">%s\n%s\n" % (sid.replace(":","-"), conseqs[j].replace(".", "-")))
 
-        clonef.write(">%s_GERM\n" % clones[0].clone)
+        germ_id = ["GERM"]
+        if meta_data is not None:
+            for i in range(1,len(meta_data)):
+                germ_id.append("GERM")
+        clonef.write(">%s_%s\n" % (clones[0].clone,"_".join(germ_id)))
         for i in range(0, len(newgerm)):
             clonef.write("%s" % newgerm[i].replace(".","-"))
         clonef.write("\n")
@@ -572,7 +626,7 @@ def outputSeqPartFiles(out_dir, useqs_f, meta_data, clones, collapse, nseqs, del
         partf.write("\n")
 
 
-def outputIgPhyML(clones, sequences, meta_data=None, collapse=False, logs=None,
+def outputIgPhyML(clones, sequences, meta_data=None, collapse=False, ncdr3=False, logs=None,
                   fail_writer=None, out_dir=None, min_seq=1):
     """
     Create intermediate sequence alignment and partition files for IgPhyML output
@@ -582,6 +636,7 @@ def outputIgPhyML(clones, sequences, meta_data=None, collapse=False, logs=None,
       sequences (list): sequences within the same clone (share indexes with clones parameter).
       meta_data (str): Field to append to sequence IDs. Splits identical sequences with different meta_data
       collapse (bool): if True collapse identical sequences.
+      ncdr3 (bool): if True remove CDR3
       logs (dict): contains log information for each sequence
       out_dir (str): directory for output files.
       fail_writer (changeo.IO.TSVWriter): failed sequences writer object.
@@ -624,14 +679,36 @@ def outputIgPhyML(clones, sequences, meta_data=None, collapse=False, logs=None,
     elif len(lcodon) == 1:
         newgerm[-1] = newgerm[-1] + "NN"
 
+    if ncdr3:
+        ngerm = []
+        nimgt = []
+        for i in range(0, len(newseqs)):
+            nseq = []
+            ncdr3 = 0
+            for j in range(0, len(imgt)):
+                if imgt[j] != 108:
+                    nseq.append(newseqs[i][j])
+                    if i == 0:
+                        ngerm.append(newgerm[j])
+                        nimgt.append(imgt[j])
+                else:
+                    ncdr3 += 1
+            newseqs[i] = nseq
+        newgerm = ngerm
+        imgt = nimgt
+            #print("Length: " + str(ncdr3))
+
     useqs_f = OrderedDict()
     conseqs = []
     for j in range(0, nseqs):
         conseq = "".join([str(seq_rec) for seq_rec in newseqs[j]])
         if meta_data is not None:
-            if isinstance(clones[j].getField(meta_data[0]), str):
-                clones[j].setField(meta_data[0],clones[j].getField(meta_data[0]).replace("_", ""))
-            conseq_f = "".join([str(seq_rec) for seq_rec in newseqs[j]])+delim+str(clones[j].getField(meta_data[0]))
+            meta_data_list = []
+            for m in range(0,len(meta_data)):
+                if isinstance(clones[j].getField(meta_data[m]), str):
+                    clones[j].setField(meta_data[m],clones[j].getField(meta_data[m]).replace("_", ""))
+                meta_data_list.append(str(clones[j].getField(meta_data[m])))
+            conseq_f = "".join([str(seq_rec) for seq_rec in newseqs[j]])+delim+":".join(meta_data_list)
         else:
             conseq_f = conseq
         if conseq_f in useqs_f and collapse:
@@ -679,9 +756,12 @@ def maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer):
       cloneseqs (list): list of masked clone sequences.
       logs (dict): contains log information for each sequence.
       fails (dict): counts of various sequence processing failures.
+      out_args (dict): arguments for output preferences.
+      fail_writer (changeo.IO.TSVWriter): failed sequences writer object.
 
     Returns:
-      None: returns None if an error occurs.
+      0: returns 0 if an error occurs or masking fails.
+      1: returns 1 masking succeeds
     """
     if r.clone is None:
         printError("Cannot export datasets until sequences are clustered into clones.")
@@ -703,13 +783,13 @@ def maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer):
         logs[r.sequence_id]["FAIL"] = "Germline PTC"
         fails["seq_fail"] += 1
         fails["germlineptc"] += 1
-        return None
+        return 0
 
     if r.functional and ptcs < 0:
         #If IMGT regions are provided, record their positions
         regions = getRegions(r.sequence_imgt, r.junction_length)
         #print(regions["cdr1_imgt"]+regions["fwr4_imgt"])
-        if regions["cdr3_imgt"] is not "":
+        if regions["cdr3_imgt"] is not "" and regions["cdr3_imgt"] is not None:
             simgt = regions["fwr1_imgt"] + regions["cdr1_imgt"] + regions["fwr2_imgt"] + regions["cdr2_imgt"] + \
                     regions["fwr3_imgt"] + regions["cdr3_imgt"] + regions["fwr4_imgt"]
             if len(simgt) < len(r.sequence_imgt):
@@ -733,12 +813,11 @@ def maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer):
                 logs[r.sequence_id]["FWRCDRSEQ"] = simgt
                 fails["seq_fail"] += 1
                 fails["region_fail"] += 1
-                return None
+                return 0
         else:
             #imgt_warn = "\n! IMGT FWR/CDR sequence columns not detected.\n! Cannot run CDR/FWR partitioned model on this data.\n"
             imgtpartlabels = [0] * len(r.sequence_imgt)
             r.setField("imgtpartlabels", imgtpartlabels)
-        #print(r.sequence_imgt)
         mout = maskSplitCodons(r)
         mask_seq = mout[0]
         ptcs = hasPTC(mask_seq)
@@ -755,6 +834,7 @@ def maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer):
             else:
                 clones[r.clone] = [r]
                 cloneseqs[r.clone] = [mask_seq]
+            return 1
         else:
             if out_args["failed"]:
                 fail_writer.writeReceptor(r)
@@ -779,16 +859,140 @@ def maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer):
         fails["seq_fail"] += 1
         fails["nf_fail"] += 1
 
+    return 0
+
+# Run IgPhyML on outputed data
+def runIgPhyML(outfile, igphyml_out, clone_dir, nproc=1, optimization="lr", omega="e,e", kappa="e", motifs="FCH",
+               hotness="e,e,e,e,e,e",oformat="tab", nohlp=False, clean="none"):
+    """
+    Run IgPhyML on outputted data
+
+    Arguments:
+      outfile (str): Output file name.
+      igphymlout (str): igphyml output file
+      nproc (int): Number of threads to parallelize IgPhyML across
+      optimization (str): Optimize combination of topology (t) branch lengths (l) and parameters (r) in IgPhyML.
+      omega (str): omega optimization in IgPhyML (--omega)
+      kappa (str): kappa optimization in IgPhyML (-t)
+      motifs (str): motifs to use in IgPhyML (--motifs)
+      hotness (str): motif in IgPhyML (--hotness)
+      oformat (str): output format for IgPhyML (tab or txt)
+      nohlp (bool): If True, only estimate GY94 trees and parameters
+      clean (str): delete intermediate files? (none, all)
+    """
+    osplit = outfile.split(".")
+    outrep = ".".join(osplit[0:(len(osplit)-1)]) + "_gy.tsv"
+    gyout = outfile + "_igphyml_stats_gy.txt"
+
+    gy_args = ["igphyml", "--repfile", outfile, "-m", "GY", "--run_id", "gy", "--outrep", outrep, "--threads",
+               str(nproc),"--outname",gyout]
+
+
+    hlp_args = ["igphyml","--repfile", outrep, "-m", "HLP", "--run_id", "hlp", "--threads", str(nproc), "-o",
+                optimization, "--omega", omega, "-t", kappa, "--motifs", motifs, "--hotness", hotness, "--oformat",
+                oformat, "--outname", igphyml_out]
+
+    log = OrderedDict()
+    log["START"] = "IgPhyML GY94 tree estimation"
+    printLog(log)
+
+    try: #check for igphyml executable
+        subprocess.check_output(["igphyml"])
+    except:
+        printError("igphyml not found :-/")
+    try: #get GY94 starting topologies
+        p = subprocess.check_output(gy_args)
+    except subprocess.CalledProcessError as e:
+        print(" ".join(gy_args))
+        print('error>', e.output, '<')
+        printError("GY94 tree building in IgPhyML failed")
+
+    log = OrderedDict()
+    log["START"] = "IgPhyML HLP analysis"
+    log["OPTIMIZE"] = optimization
+    log["TS/TV"] = kappa
+    log["wFWR,wCDR"] = omega
+    log["MOTIFS"] = motifs
+    log["HOTNESS"] = hotness
+    log["NPROC"] = nproc
+    printLog(log)
+
+    if not nohlp:
+        try: #estimate HLP parameters/trees
+            p = subprocess.check_output(hlp_args)
+        except subprocess.CalledProcessError as e:
+            print(" ".join(hlp_args))
+            print('error>', e.output, '<')
+            printError("HLP tree building failed")
+
+    log = OrderedDict()
+    log["OUTPUT"] = igphyml_out
+    if oformat == "tab":
+        igf = open(igphyml_out)
+        names = igf.readline().split("\t")
+        vals = igf.readline().split("\t")
+        for i in range(3,len(names)-1):
+            log[names[i]] = round(float(vals[i]),2)
+    printLog(log)
+
+    if clean != "none":
+        log = OrderedDict()
+        log["START"] = "CLEANING"
+        log["SCOPE"] = clean
+        printLog(log)
+        todelete = open(outrep)
+        for line in todelete:
+            line = line.rstrip("\n")
+            line = line.rstrip("\r")
+            lsplit = line.split("\t")
+            if len(lsplit) == 4:
+                os.remove(lsplit[0])
+                os.remove(lsplit[1])
+                os.remove(lsplit[3])
+        todelete.close()
+        os.remove(outrep)
+        os.remove(outfile)
+        os.remove(gyout)
+        cilog = outrep + "_igphyml_CIlog.txt_hlp"
+        if os.path.isfile(cilog):
+            os.remove(cilog)
+        if oformat == "tab":
+            os.rmdir(clone_dir)
+        else:
+            printWarning("Using --clean all with --oformat txt will delete all tree file results.\n"
+                         "You'll have to do that yourself.")
+        log = OrderedDict()
+        log["END"] = "IgPhyML analysis"
+        printLog(log)
+
 
 # Note: Collapse can give misleading dupcount information if some sequences have ambiguous characters at polymorphic sites
-def buildTrees(db_file, meta_data=None, collapse=False, min_seq=1, format=default_format, out_args=default_out_args):
+def buildTrees(db_file, meta_data=None, target_clones=None, collapse=False, ncdr3=False, sample_depth=-1, min_seq=1,append=None,
+               igphyml=False, nproc=1, optimization="lr", omega="e,e", kappa="e", motifs="FCH",
+               hotness="e,e,e,e,e,e", oformat="tab", clean="none", nohlp=False,
+               format=default_format, out_args=default_out_args):
     """
     Masks codons split by alignment to IMGT reference, then produces input files for IgPhyML
 
     Arguments:
       db_file (str): input tab-delimited database file.
       meta_data (str): Field to append to sequence IDs. Splits identical sequences with different meta_data
+      target_clones (str): List of clone IDs to analyze.
       collapse (bool): if True collapse identical sequences.
+      ncdr3 (bool): if True remove all CDR3s.
+      sample_depth (int): depth of subsampling before deduplication
+      min_seq (int): minimum number of sequences per clone
+      append (str): column name to append to sequence_id
+      igphyml (bool): If True, run IgPhyML on outputted data
+      nproc (int) : Number of threads to parallelize IgPhyML across
+      optimization (str): Optimize combination of topology (t) branch lengths (l) and parameters (r) in IgPhyML.
+      omega (str): omega optimization in IgPhyML (--omega)
+      kappa (str): kappa optimization in IgPhyML (-t)
+      motifs (str): motifs to use in IgPhyML (--motifs)
+      hotness (str): motif in IgPhyML (--hotness)
+      oformat (str): output format for IgPhyML (tab or txt)
+      clean (str): delete intermediate files? (none, all)
+      nohlp (bool): If True, only estimate GY94 trees and parameters
       format (str): input and output format.
       out_args (dict): arguments for output preferences.
 
@@ -803,19 +1007,27 @@ def buildTrees(db_file, meta_data=None, collapse=False, min_seq=1, format=defaul
     printLog(log)
 
     # Open output files
+    out_label = "lineages"
+
     pass_handle = getOutputHandle(db_file,
-                                  out_label="lineages",
+                                  out_label=out_label,
                                   out_dir=out_args["out_dir"],
-                                  out_name=out_args["out_name"],
+                                  out_name= out_args["out_name"],
                                   out_type="tsv")
 
+    igphyml_out = None
+    if igphyml:
+        igphyml_out = getOutputName(db_file, out_label="igphyml-pass",
+                                    out_dir=out_args["out_dir"],
+                                    out_name=out_args["out_name"],
+                                    out_type=oformat)
+
     dir_name, __ = os.path.split(pass_handle.name)
 
     if out_args["out_name"] is None:
         __, clone_name, __ = splitName(db_file)
     else:
         clone_name = out_args["out_name"]
-    # clone_dir = outdir/out_name
     if dir_name is None:
         clone_dir = clone_name
     else:
@@ -853,8 +1065,52 @@ def buildTrees(db_file, meta_data=None, collapse=False, min_seq=1, format=defaul
     # Mask codons split by indels
     start_time = time()
     printMessage("Correcting frames and indels of sequences", start_time=start_time, width=50)
+
+    #subsampling loop
+    init_clone_sizes = {}
+    big_enough = []
+    all_records = []
+    found_no_funct = False
     for r in records:
-        maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer)
+        if r.functional is None:
+            r.functional = True
+            if found_no_funct is False:
+                printWarning("FUNCTIONAL column not found.")
+                found_no_funct = True
+        all_records.append(r)
+        if r.clone in init_clone_sizes:
+            init_clone_sizes[r.clone] += 1
+        else:
+            init_clone_sizes[r.clone] = 1
+
+    for r in all_records:
+        if target_clones is None or r.clone in target_clones:
+            if init_clone_sizes[r.clone] >= min_seq:
+               big_enough.append(r)
+
+    if len(big_enough) == 0:
+        printError("\n\nNo sequences found that match specified criteria.",1)
+
+    if sample_depth > 0:
+        random.shuffle(big_enough)
+
+    total = 0
+    for r in big_enough:
+        if r.functional is None:
+            r.functional = True
+            if found_no_funct is False:
+                printWarning("FUNCTIONAL column not found.")
+                found_no_funct = True
+
+        r.sequence_id = r.sequence_id.replace(",","-") #remove commas from sequence ID
+        r.sequence_id = r.sequence_id.replace(":","-") #remove colons from sequence ID
+        if append is not None:
+            if append is not None:
+                for m in append:
+                    r.sequence_id = r.sequence_id + "_" + r.getField(m)
+        total += maskCodonsLoop(r, clones, cloneseqs, logs, fails, out_args, fail_writer)
+        if total == sample_depth:
+            break
 
     # Start processing clones
     clonesizes = {}
@@ -868,7 +1124,9 @@ def buildTrees(db_file, meta_data=None, collapse=False, min_seq=1, format=defaul
             clonesizes[str(k)] = -len(cloneseqs[str(k)])
         else:
             clonesizes[str(k)] = outputIgPhyML(clones[str(k)], cloneseqs[str(k)], meta_data=meta_data, collapse=collapse,
-                                           logs=logs, fail_writer=fail_writer, out_dir=clone_dir, min_seq=min_seq)
+                                           ncdr3=ncdr3, logs=logs, fail_writer=fail_writer,
+                                           out_dir=clone_dir, min_seq=min_seq)
+
         #If clone is too small, size is returned as a negative
         if clonesizes[str(k)] > 0:
             nclones += 1
@@ -893,7 +1151,11 @@ def buildTrees(db_file, meta_data=None, collapse=False, min_seq=1, format=defaul
         outfile = os.path.join(clone_dir, "%s.fasta" % key)
         partfile = os.path.join(clone_dir, "%s.part.txt" % key)
         if clonesizes[key] > 0:
-            pass_handle.write("%s\t%s\t%s\t%s\n" % (outfile, "N", key+"_GERM", partfile))
+            germ_id = ["GERM"]
+            if meta_data is not None:
+                for i in range(1, len(meta_data)):
+                    germ_id.append("GERM")
+            pass_handle.write("%s\t%s\t%s_%s\t%s\n" % (outfile, "N", key,"_".join(germ_id), partfile))
 
     handle.close()
     output = {"pass": None, "fail": None}
@@ -925,8 +1187,15 @@ def buildTrees(db_file, meta_data=None, collapse=False, min_seq=1, format=defaul
     log["END"] = "BuildTrees"
     printLog(log)
 
+    #Run IgPhyML on outputted data?
+    if igphyml:
+        runIgPhyML(pass_handle.name, igphyml_out=igphyml_out, clone_dir=clone_dir, nproc=nproc,
+                   optimization=optimization, omega=omega, kappa=kappa, motifs=motifs,
+                   hotness=hotness, oformat=oformat, nohlp=nohlp,clean=clean)
+
     return output
 
+
 def getArgParser():
     """
     Defines the ArgumentParser
@@ -944,29 +1213,73 @@ def getArgParser():
                     successfully processed records.
                  lineages-fail
                     database records failed processing.
+                 igphyml-pass
+                    parameter estimates and lineage trees from running IgPhyML, if specified
 
              required fields:
                  SEQUENCE_ID, SEQUENCE_INPUT, SEQUENCE_IMGT,
-                 GERMLINE_IMGT_D_MASK, V_CALL, J_CALL
+                 GERMLINE_IMGT_D_MASK, V_CALL, J_CALL, CLONE,
+                 V_SEQ_START
               """)
 
     # Parent parser
-    parser_parent = getCommonArgParser(out_file=False, log=True, format=False)
+    parser_parent = getCommonArgParser(out_file=False, log=True, format=True)
 
     # Define argument parser
     parser = ArgumentParser(description=__doc__, epilog=fields,
                             parents=[parser_parent],
                             formatter_class=CommonHelpFormatter, add_help=False)
 
-    group = parser.add_argument_group("tree building arguments")
+    group = parser.add_argument_group("sequence processing arguments")
     group.add_argument("--collapse", action="store_true", dest="collapse",
                         help="""If specified, collapse identical sequences before exporting to fasta.""")
+    group.add_argument("--ncdr3", action="store_true", dest="ncdr3",
+                        help="""If specified, remove CDR3 from all sequences.""")
     group.add_argument("--md", nargs="+", action="store", dest="meta_data",
                        help="""List of fields to containing metadata to include in output fasta file 
                             sequence headers.""")
+    group.add_argument("--clones", nargs="+", action="store", dest="target_clones",
+                       help="""List of clone IDs to output, if specified.""")
     group.add_argument("--minseq", action="store", dest="min_seq", type=int, default=1,
                        help="""Minimum number of data sequences. Any clones with fewer than the specified
                             number of sequences will be excluded.""")
+    group.add_argument("--sample", action="store", dest="sample_depth", type=int, default=-1,
+                       help="""Depth of reads to be subsampled (before deduplication).""")
+    group.add_argument("--append", nargs="+", action="store", dest="append",
+                       help="""List of columns to append to sequence ID to ensure uniqueness.""")
+
+    igphyml_group = parser.add_argument_group("IgPhyML arguments (see igphyml -h for details)")
+    igphyml_group.add_argument("--igphyml", action="store_true", dest="igphyml",
+                               help="""Run IgPhyML on output?""")
+    igphyml_group.add_argument("--nproc", action="store", dest="nproc", type=int, default=1,
+                               help="""Number of threads to parallelize IgPhyML across.""")
+    igphyml_group.add_argument("--clean", action="store", choices=("none", "all"),
+                               dest="clean", type=str, default="none",
+                               help="""Delete intermediate files? 
+                               none: leave all intermediate files; all: delete all intermediate files.""")
+    igphyml_group.add_argument("--optimize", action="store", dest="optimization", type=str, default="lr",
+                               choices=("n","r","l","lr","tl","tlr"),
+                               help="""Optimize combination of topology (t) branch lengths (l) and parameters (r), or 
+                               nothing (n), for IgPhyML.""")
+    igphyml_group.add_argument("--omega", action="store", dest="omega", type=str, default="e,e",
+                               choices = ("e", "ce", "e,e", "ce,e", "e,ce", "ce,ce"),
+                               help="""Omega parameters to estimate for FWR,CDR respectively: 
+                               e = estimate, ce = estimate + confidence interval""")
+    igphyml_group.add_argument("-t", action="store", dest="kappa", type=str, default="e",
+                               choices=("e", "ce"),
+                               help="""Kappa parameters to estimate: 
+                               e = estimate, ce = estimate + confidence interval""")
+    igphyml_group.add_argument("--motifs", action="store", dest="motifs", type=str,
+                               default="WRC_2:0,GYW_0:1,WA_1:2,TW_0:3,SYC_2:4,GRS_0:5",
+                               help="""Which motifs to estimate mutability.""")
+    igphyml_group.add_argument("--hotness", action="store", dest="hotness", type=str, default="e,e,e,e,e,e",
+                               help="""Mutability parameters to estimate: 
+                               e = estimate, ce = estimate + confidence interval""")
+    igphyml_group.add_argument("--oformat", action="store", dest="oformat", type=str, default="tab",
+                               choices=("tab", "txt"),
+                               help="""IgPhyML output format.""")
+    igphyml_group.add_argument("--nohlp", action="store_true", dest="nohlp",
+                               help="""Don't run HLP model?""")
 
     return parser
 
@@ -985,4 +1298,4 @@ if __name__ == "__main__":
     # Call main for each input file
     for f in args.__dict__["db_files"]:
         args_dict["db_file"] = f
-        buildTrees(**args_dict)
+        buildTrees(**args_dict)
\ No newline at end of file
