@@ -32,7 +32,7 @@ from changeo.Gene import allele_number_regex, allele_regex, gene_regex, family_r
 #           changeo.Receptor.Schema
 #         """
 #         with resource_stream(__name__, 'data/receptor.yaml') as f:
-#             data = yaml.load(f)
+#             data = yaml.load(f, Loader=yaml.FullLoader)
 #             receptor = {v[schema]: k for k, v in data['receptor'].items()}
 #             definition = data[schema]
 #
@@ -92,6 +92,7 @@ class AIRRSchema:
                                  ('productive', 'functional'),
                                  ('stop_codon', 'stop'),
                                  ('vj_in_frame', 'in_frame'),
+                                 ('locus', 'locus'),
                                  ('v_call', 'v_call'),
                                  ('d_call', 'd_call'),
                                  ('j_call', 'j_call'),
@@ -116,7 +117,6 @@ class AIRRSchema:
 
     # Custom fields
     _custom_map = OrderedDict([('c_call', 'c_call'),
-                               ('locus', 'locus'),
                                ('germline_alignment_d_mask', 'germline_imgt_d_mask'),
                                ('v_score', 'v_score'),
                                ('v_identity', 'v_identity'),
@@ -138,6 +138,8 @@ class AIRRSchema:
                                ('fwr2', 'fwr2_imgt'),
                                ('fwr3', 'fwr3_imgt'),
                                ('fwr4', 'fwr4_imgt'),
+                               ('junction_start', 'junction_start'),
+                               ('junction_end', 'junction_end'),
                                ('cdr1_start', 'cdr1_start'),
                                ('cdr1_end', 'cdr1_end'),
                                ('cdr2_start', 'cdr2_start'),
@@ -163,6 +165,7 @@ class AIRRSchema:
                                ('cdr3_igblast_aa', 'cdr3_igblast_aa'),
                                ('duplicate_count', 'dupcount'),
                                ('consensus_count', 'conscount'),
+                               ('umi_count', 'umicount'),
                                ('clone_id', 'clone'),
                                ('cell_id', 'cell')])
     custom_fields = list(_custom_map.keys())
@@ -219,6 +222,7 @@ class ChangeoSchema:
                                  ('STOP', 'stop'),
                                  ('MUTATED_INVARIANT', 'mutated_invariant'),
                                  ('INDELS', 'indels'),
+                                 ('LOCUS', 'locus'),
                                  ('V_CALL', 'v_call'),
                                  ('D_CALL', 'd_call'),
                                  ('J_CALL', 'j_call'),
@@ -246,7 +250,8 @@ class ChangeoSchema:
     standard_fields = list(_standard_map.keys())
 
     # Custom fields
-    _custom_map = OrderedDict([('V_SCORE', 'v_score'),
+    _custom_map = OrderedDict([('JUNCTION_START', 'junction_start'),
+                               ('V_SCORE', 'v_score'),
                                ('V_IDENTITY', 'v_identity'),
                                ('V_EVALUE', 'v_evalue'),
                                ('V_BTOP', 'v_btop'),
@@ -276,10 +281,12 @@ class ChangeoSchema:
                                ('P3D_LENGTH', 'p3d_length'),
                                ('P5J_LENGTH', 'p5j_length'),
                                ('D_FRAME', 'd_frame'),
+                               ('C_CALL', 'c_call'),
                                ('CDR3_IGBLAST', 'cdr3_igblast'),
                                ('CDR3_IGBLAST_AA', 'cdr3_igblast_aa'),
                                ('CONSCOUNT', 'conscount'),
                                ('DUPCOUNT', 'dupcount'),
+                               ('UMICOUNT', 'umicount'),
                                ('CLONE', 'clone'),
                                ('CELL', 'cell')])
     custom_fields = list(_custom_map.keys())
@@ -340,6 +347,7 @@ class ReceptorData:
 
       junction (Bio.Seq.Seq): ungapped junction region nucletide sequence.
       junction_aa (Bio.Seq.Seq): ungapped junction region amino acid sequence.
+      junction_start (int): start positions of the junction in the input nucleotide sequence.
       junction_length (int): length of the junction in nucleotides.
 
       germline_vdj (Bio.Seq.Seq): full ungapped germline V(D)J nucleotide sequence.
@@ -415,6 +423,7 @@ class ReceptorData:
 
       conscount (int): number of reads contributing to the UMI consensus sequence.
       dupcount (int): copy number of the sequence.
+      umicount (int): number of UMIs representing the sequence.
 
       clone (str): clonal cluster identifier.
       cell (str): origin cell identifier.
@@ -422,7 +431,7 @@ class ReceptorData:
       annotations (dict): dictionary containing all unknown fields.
     """
     #with resource_stream(__name__, 'data/receptor.yaml') as f:
-    #    data = yaml.load(f)
+    #    data = yaml.load(f, Loader=yaml.FullLoader)
     #
     # # Define type parsers
     # parsers = {k: v['type'] for k, v in data['receptor'].items()}
@@ -458,6 +467,7 @@ class ReceptorData:
                'sequence_vdj': 'nucleotide',
                'junction': 'nucleotide',
                'junction_aa': 'aminoacid',
+               'junction_start': 'integer',
                'junction_length': 'integer',
                'germline_imgt': 'nucleotide',
                'germline_imgt_d_mask': 'nucleotide',
@@ -519,6 +529,7 @@ class ReceptorData:
                'cdr3_igblast_aa': 'aminoacid',
                'conscount': 'integer',
                'dupcount': 'integer',
+               'umicount': 'integer',
                'clone': 'identity',
                'cell': 'identity'}
 
@@ -533,6 +544,7 @@ class ReceptorData:
                        ('d_germ_start', 'd_germ_length', 'd_germ_end'),
                        ('j_seq_start', 'j_seq_length', 'j_seq_end'),
                        ('j_germ_start', 'j_germ_length', 'j_germ_end'),
+                       ('junction_start', 'junction_length', 'junction_end'),
                        ('fwr1_start', 'fwr1_length', 'fwr1_end'),
                        ('fwr2_start', 'fwr2_length', 'fwr2_end'),
                        ('fwr3_start', 'fwr3_length', 'fwr3_end'),
@@ -619,10 +631,21 @@ class Receptor:
     _derived = {'v_seq_end': 'integer',
                 'v_germ_end_vdj': 'integer',
                 'v_germ_end_imgt': 'integer',
+                'd_seq_end': 'integer',
+                'd_germ_end': 'integer',
                 'j_seq_end': 'integer',
                 'j_germ_end': 'integer',
-                'd_seq_end': 'integer',
-                'd_germ_end': 'integer'}
+                'junction_end': 'integer'}
+
+    def _junction_start(self):
+        """
+        Determine the position of the first junction nucleotide in the input sequence
+        """
+        try:
+            x = self.v_germ_end_imgt - 310
+            return self.v_seq_end - x if x >= 0 else None
+        except TypeError:
+            return None
 
     def __init__(self, data):
         """
@@ -654,6 +677,10 @@ class Receptor:
             f = getattr(ReceptorData, ReceptorData.parsers[k])
             setattr(self, k, f(data.pop(k, None)))
 
+        # Derive junction_start if not provided
+        if not hasattr(self, 'junction_start'):
+            setattr(self, 'junction_start', self._junction_start(self))
+
         # Add remaining elements as annotations dictionary
         self.annotations = data
 
@@ -1055,7 +1082,7 @@ class Receptor:
     @property
     def v_seq_end(self):
         """
-        position of the last V nucleotide in the input sequence.
+        Position of the last V nucleotide in the input sequence
         """
         try:  return self.v_seq_start + self.v_seq_length - 1
         except TypeError:  return None
@@ -1063,7 +1090,7 @@ class Receptor:
     @property
     def v_germ_end_imgt(self):
         """
-        position of the last nucleotide in the IMGT-gapped V germline sequence alignment.
+        Position of the last nucleotide in the IMGT-gapped V germline sequence alignment
         """
         try:  return self.v_germ_start_imgt + self.v_germ_length_imgt - 1
         except TypeError:  return None
@@ -1071,7 +1098,7 @@ class Receptor:
     @property
     def v_germ_end_vdj(self):
         """
-        position of the last nucleotide in the ungapped V germline sequence alignment.
+        Position of the last nucleotide in the ungapped V germline sequence alignment
         """
         try:  return self.v_germ_start_vdj + self.v_germ_length_vdj - 1
         except TypeError:  return None
@@ -1079,7 +1106,7 @@ class Receptor:
     @property
     def d_seq_end(self):
         """
-        position of the last D nucleotide in the input sequence.
+        Position of the last D nucleotide in the input sequence
         """
         try:  return self.d_seq_start + self.d_seq_length - 1
         except TypeError:  return None
@@ -1087,7 +1114,7 @@ class Receptor:
     @property
     def d_germ_end(self):
         """
-        position of the last nucleotide in the D germline sequence alignment.
+        Position of the last nucleotide in the D germline sequence alignment
         """
         try:  return self.d_germ_start + self.d_germ_length - 1
         except TypeError:  return None
@@ -1095,7 +1122,7 @@ class Receptor:
     @property
     def j_seq_end(self):
         """
-        position of the last J nucleotide in the input sequence.
+        Position of the last J nucleotide in the input sequence
         """
         try:  return self.j_seq_start + self.j_seq_length - 1
         except TypeError:  return None
@@ -1103,26 +1130,15 @@ class Receptor:
     @property
     def j_germ_end(self):
         """
-        position of the last nucleotide in the J germline sequence alignment.
+        Position of the last nucleotide in the J germline sequence alignment
         """
         try:  return self.j_germ_start + self.j_germ_length - 1
         except TypeError:  return None
 
-    @property
-    def junction_start(self):
-        """
-        position of the first junction nucleotide in the input sequence.
-        """
-        try:
-            x = self.v_germ_end_imgt - 310
-            return self.v_seq_end - x if x >= 0 else None
-        except TypeError:
-            return None
-
     @property
     def junction_end(self):
         """
-        position of the last junction nucleotide in the input sequence.
+        Position of the last junction nucleotide in the input sequence
         """
         try:
             gaps = self.junction.count('.')
